#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include "config.h"

static FILE *next_file(FILE *old, int x, int slen, char *prefix) {
	if (old != NULL)
		fclose(old);

	/* Gen file name */
	char name[BUF_SIZE + 1];
	int len = snprintf(name, sizeof(name), "%s", prefix);

	for (int i = slen; i >= 0; i--) {
		if (len + i >= (int)(BUF_SIZE * sizeof(char)))
			break;

		name[len + i] = 'a' + (x % 26);
		x /= 26;
	}

	/* Open file */
	FILE *fp = fopen(name, "w");
	if (fp == NULL) {
		fprintf(stderr, "split: %s\n", strerror(errno));
		return NULL;
	}

	return fp;
}

int main(int argc, char **argv) {
	off_t size = 0;
	int a_flag = 1;
	int b_flag = 0;
	char *prefix = "x";

	int opt;
	while ((opt = getopt(argc, argv, "l:a:b:")) != -1) {
		switch (opt) {
			case 'a':
				a_flag = atoi(optarg);
				break;

			case 'l':
				size = atoi(optarg);
				break;

			case 'b':
				switch (optarg[strlen(optarg) - 1]) {
					case 'm':
						sscanf(optarg, "%ld", &size);
						size *= 1048576;
						break;

					case 'k':
						sscanf(optarg, "%ld", &size);
						size *= 1024;
						break;

					default:
						sscanf(optarg, "%ld", &size);
						break;
				}

				b_flag = 1;
				break;

			default:
				puts("split [alb] [file1]\n\t-l N Split by N lines\n\t-a N Use N letters as prefix\n\t-b N[k|m] Split by N (kilo|mega)bytes");
				return 0;
		}
	}

	argv += optind;
	argc -= optind;

	FILE *fp = stdin;
	if (argv[0] != NULL && strcmp(argv[0], "-")) {
		fp = fopen(argv[0], "r");
		if (fp == NULL) {
			fprintf(stderr, "split: %s: %s\n", argv[0], strerror(errno));
			return 1;
		}
	}

	if (argc == 2 && argv[1] != NULL)
		prefix = argv[1];


	int ret = 0;
	int files = 0;
	FILE *out = NULL;

	off_t n = 0;
	int ch;
	while ((ch = getc(fp)) != EOF) {
		if (out == NULL || n >= size) {
			out = next_file(out, files++, a_flag, prefix);
			if (out == NULL) {
				ret = 1;
				break;
			}

			n = 0;
		}

		if (ch == '\n' || b_flag)
			n++;

		putc(ch, out);
	}

	fclose(fp);
	return ret;
}
